#include <PalmOS.h>
#include <Table.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

//---------------------------------------
// Initialize the Artist Table structure
//---------------------------------------

void InitArtistTable(TableType *table) {
	UInt32 i;

	for(i = 0; i < artistVisibleRows; i++)
		TblSetItemStyle(table, i, 0, customTableItem);
	TblSetColumnUsable(table, 0, true);
	TblSetCustomDrawProcedure(table, 0, DrawArtistRow);
}

//-------------------------------------
// Populate the Artist Table structure
//-------------------------------------

void PopulateArtistTable(TableType *table, UInt32 artistIndex, Boolean draw) {
	MemHandle aItem;
	ArtistRecord *aRec;
	UInt32 i;

	artistIndex += DBArtistRecordStart;

	for(i = 0; i < artistVisibleRows; i++) {
		if(artistIndex <= DBArtistRecordLength) {
			aItem = DmQueryRecord(palantirDB, artistIndex);
			aRec = (ArtistRecord *) MemHandleLock(aItem);
			MemHandleUnlock(aItem);
			TblSetItemPtr(table, i, 0, aRec);
			artistIndex++;
		} else {
			TblSetRowUsable(table, i, false);
		}
	}

	if(draw) TblDrawTable(table);
}

//-------------------------
// Scroll the Artist Table
//-------------------------

void ScrollArtistTable(Int32 newValue, Boolean setBar) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, ArtistTable));
	Int32 maxValue = DBArtistRecordLength - artistVisibleRows;
	if(maxValue < 0)
		maxValue = 0;	
	//make sure we don't scroll above 0 or below max
	if(newValue > maxValue)
		newValue = maxValue;
	if(newValue < 0)
		newValue = 0;

	artistTablePosition = newValue;
	PopulateArtistTable(table, artistTablePosition, true);
	if(setBar)
		SclSetScrollBar(artistScrollBar, artistTablePosition, 0, maxValue, artistVisibleRows);
}

//----------------------------------------
// Initialize the Artist Table scrollbars
//----------------------------------------

void InitArtistScrollBar() {
	Int16 max;
	if(artistVisibleRows > DBArtistRecordLength)
		max = 0;
	else
		max = DBArtistRecordLength - artistVisibleRows;
	SclSetScrollBar(artistScrollBar, artistTablePosition, 0, max, artistVisibleRows);
}

//----------------------------
// Initialize the Artist View
//----------------------------

void ArtistFormInit(void) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, ArtistTable));

	//set up some global variables
	lastScreen = ARTIST_SCREEN;
	artistVisibleRows = TblGetNumberOfRows(table);	//set the number of rows in the table
	artistScrollBar = (ScrollBarType *) FrmGetObjectPtr(form, FrmGetObjectIndex(form, ArtistScrollBar));
	sourceTablePosition = 0;			//when we select an artist, first set the source at the top
	
	//set the artist scroll bar object
	InitArtistTable(table);
	InitArtistScrollBar();
	PopulateArtistTable(table, artistTablePosition, false);
	FrmDrawForm(form);
}

//---------------------------
// Handle events in ArtistForm
//---------------------------

Boolean ArtistFormHandleEvent(EventType *event) {
	Int16 row;
	Boolean handled = false;
	TableType *table;
	ArtistRecord *aRec;

	switch(event->eType) {
	case frmOpenEvent:
		//FrmDrawForm(form);
		ArtistFormInit();
		handled = true;
		break;
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		case ArtistButtonPlaylist:
			FrmGotoForm(PlaylistForm);
			handled = true;
			break;
		default:
			break;
		}
		break;
	case menuEvent:
		handled = GlobalMenuHandleEvent(event->data.menu.itemID);
		break;
	case sclRepeatEvent:
		ScrollArtistTable(event->data.sclExit.newValue, false);
		break;
	case keyDownEvent:
		if(event->data.keyDown.chr == vchrPageDown)
			ScrollArtistTable(artistTablePosition + artistVisibleRows, true);
		if(event->data.keyDown.chr == vchrPageUp)
			ScrollArtistTable(artistTablePosition - artistVisibleRows, true);
		handled = true;
		break;
	case tblSelectEvent:
		row = event->data.tblEnter.row;
		table = event->data.tblEnter.pTable;
		aRec = TblGetItemPtr(table, row, 0);
		artistSelected = aRec;
		FrmGotoForm(SourceForm);
		handled = true;
		break;
	default:
		break;
	}
	return handled;
}
