#include <PalmOS.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

//------------------------
// Unpack the TrackRecord
//------------------------
void UnpackTrackRecord(TrackRecord *record, const TrackRecordPacked *recordPacked) {
	const char *data = recordPacked->data;
	
	//assign the constant length vars here
	record->fid = recordPacked->fid;
	record->artistID = recordPacked->artistID;
	
	//now unpack the string data
	record->year = data;
	data += StrLen(data) + 1; record->source = data;
	data += StrLen(data) + 1; record->track = data;
	data += StrLen(data) + 1; record->title = data;
	data += StrLen(data) + 1; record->genre = data;
	data += StrLen(data) + 1; record->bitrate = data;
	data += StrLen(data) + 1; record->duration = data;
}

//----------------------
// Get the index record
//----------------------

UInt32 DatabaseGetIndexValue(DmOpenRef db, UInt32 index) {
	IndexRecord *rec;
	UInt32 returnValue = -1;
	MemHandle indexItem = DmQueryRecord(db, 0);
	
	if(indexItem) {
		rec = (IndexRecord *) MemHandleLock(indexItem);
		switch(index) {
			case HDR_ARTIST_RECORD_START:
				returnValue = rec->artistRecordStart;
				break;
			case HDR_ARTIST_RECORD_LENGTH:
				returnValue = rec->artistRecordLength;
				break;
			case HDR_TRACK_RECORD_START:
				returnValue = rec->trackRecordStart;
				break;
			case HDR_TRACK_RECORD_LENGTH:
				returnValue = rec->trackRecordLength;
				break;
			default:
		}
		MemHandleUnlock(indexItem);
	}
	return returnValue;
}

//----------------------
// Open up our database
//----------------------

Err DatabaseOpen(DmOpenRef *refPtr) {
	Err err = errNone;
	DmOpenRef ref;

	ref = DmOpenDatabaseByTypeCreator('DATA', 'EmPC', dmModeReadOnly);
	if(ref == NULL)	err = DmGetLastErr();
	*refPtr = ref;

	return err;
}
