#include <PalmOS.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

//-------------------------------
// Handle the application events
//-------------------------------

static Boolean AppHandleEvent(EventType *event) {
	FormType *form;
	UInt16 formID;
	Boolean	handled = false;

	if(event->eType == frmLoadEvent) {
		formID = event->data.frmLoad.formID;
		form = FrmInitForm(formID);
		FrmSetActiveForm(form);

		switch(formID) {
		case ArtistForm:
			FrmSetEventHandler(form, ArtistFormHandleEvent);
			break;
		case SourceForm:
			FrmSetEventHandler(form, SourceFormHandleEvent);
			break;
		case TrackForm:
			FrmSetEventHandler(form, TrackFormHandleEvent);
			break;
		case PlaylistForm:
			FrmSetEventHandler(form, PlaylistFormHandleEvent);
			break;
		default:
			break;
		}
		handled = true;
	}
	return handled;
}

//------------
// Event Loop
//------------

static void AppEventLoop(void) {
	EventType event;
	Err error;

	do {
		EvtGetEvent(&event, evtWaitForever);
		if(!SysHandleEvent(&event))
			if(!MenuHandleEvent(0, &event, &error))
				if(!AppHandleEvent(&event))
					FrmDispatchEvent(&event);
	} while(event.eType != appStopEvent);
}

//---------------------
// Application Startup
//---------------------

static Err AppStart(void) {
	Err err = errNone;

	err = DatabaseOpen(&palantirDB);
	if(err == errNone) {
		//set up some global variables
		DBArtistRecordStart = DatabaseGetIndexValue(palantirDB, HDR_ARTIST_RECORD_START);
		DBArtistRecordLength = DatabaseGetIndexValue(palantirDB, HDR_ARTIST_RECORD_LENGTH);
		DBTrackRecordStart = DatabaseGetIndexValue(palantirDB, HDR_TRACK_RECORD_START);
		DBTrackRecordLength = DatabaseGetIndexValue(palantirDB, HDR_TRACK_RECORD_LENGTH);		
		artistTablePosition = 0;		//initially set the artist table at the top
		
		//initialize the playlist
		playlistCount = 0;			//at first there are no items in the playlist
		playlist = NULL;			//at first there is nothing in the playlist
		
		FrmGotoForm(ArtistForm);
	}
	return err;
}

//------------------
// Application Stop
//------------------

static void AppStop(void) {
	//unlock the playlist if we have one
	if(playlistCount > 0) {
		MemHandle plst = MemPtrRecoverHandle(playlist);
		MemHandleUnlock(plst);
		MemHandleFree(plst);
	}
	DmCloseDatabase(palantirDB);
	FrmCloseAllForms();
}

//------------------------
// Main Function for Palm
//------------------------

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags) {
	Err error;

	switch (cmd) {
	case sysAppLaunchCmdNormalLaunch:
		error = AppStart();     // Application Startup
		if(error)
			return error;
		AppEventLoop();         // Event loop
		AppStop();              // Application Stop
		break;
	default:
		break;
	}
	return errNone;
}