#include <PalmOS.h>
#include <Table.h>
#include <ExgMgr.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

//--------------------------------
// Beam the playlist to the Empeg
//--------------------------------

void PlaylistBeam(void) {
	ExgSocketType exgSocket;
	UInt32 i, size = 0, sizeSent = 0;
	MemHandle tItem, plst = NULL;
	TrackRecordPacked *tRecPacked;
	Err err = errNone;
	Char *fids = NULL;
	Char fid[10];
	
	if(playlist != NULL) {
		for(i = 0; i < playlistCount; i++) {
			tItem = DmQueryRecord(palantirDB, playlist[i]);
			tRecPacked = (TrackRecordPacked *) MemHandleLock(tItem);
			MemHandleUnlock(tItem);
			
			StrIToH(fid, tRecPacked->fid);
			
			if(fids == NULL) {			
				plst = MemHandleNew(sizeof(Char) * (StrLen(fid) + 1));
				fids = (Char *) MemHandleLock(plst);
				StrCopy(fids, fid);
			} else {
				size = StrLen(fids) + StrLen(fid) + 1;
				MemHandleUnlock(plst);
				MemHandleResize(plst, sizeof(Char) * size);
				fids = (Char *) MemHandleLock(plst);
				StrCat(fids, fid);				
			}
		}

		MemSet(&exgSocket, sizeof(exgSocket), 0);
		size = StrLen(fids);
		//FrmCustomAlert(GlobalAlertDebug, "fids", StrIToA(charIntDebug, size), fids);
		
		exgSocket.description = "Palantir Playlist";
		exgSocket.name = "palantir_insert.empeg";
		exgSocket.length = size;
		
		err = ExgPut(&exgSocket);
		
		if (!err) {
			while (!err && sizeSent < size) {
				sizeSent += ExgSend(&exgSocket, fids, size, &err);
			}
			ExgDisconnect(&exgSocket,err);
		}

		MemHandleUnlock(plst);
		MemHandleFree(plst);
	}
}

//-------------------------------------
// Search for an index in the playlist
//-------------------------------------

Int32 PlaylistContains(UInt32 index) {
	UInt32 i;

	for(i = 0; i < playlistCount; i++)
		if(index == playlist[i]) return i;
	return -1;
}

//--------------------------------
// Add an index into the playlist
//--------------------------------

void PlaylistAdd(UInt32 index) {
	MemHandle plst;

	if(playlist == NULL) {	//we need to create a new playlist
		plst = MemHandleNew(sizeof(UInt32));
	} else {
		plst = MemPtrRecoverHandle(playlist);
		MemHandleUnlock(plst);
		MemHandleResize(plst, sizeof(UInt32) * (playlistCount + 1));
	}
	playlist = (UInt32 *)MemHandleLock(plst);
	
	playlist[playlistCount] = index;
	playlistCount++;
}

//-----------------------------------
// Remove an index from the playlist
//-----------------------------------

void PlaylistRemove(UInt32 index) {
	MemHandle plst, tmpListHandle;
	UInt32 i, idx, *tmpList;

	plst = MemPtrRecoverHandle(playlist);
	MemHandleUnlock(plst);
	playlist = (UInt32 *)MemHandleLock(plst);

	if(playlistCount > 1) {
		tmpListHandle = MemHandleNew(sizeof(UInt32) * (playlistCount - 1));
		tmpList = (UInt32 *)MemHandleLock(tmpListHandle);
		idx = PlaylistContains(index);
		for(i = 0; i < playlistCount; i++)
			if(i != idx) tmpList[i > idx ? i - 1 : i] = playlist[i];
	} else {
		tmpList = NULL;
	}

	MemHandleUnlock(plst);
	MemHandleFree(plst);
	playlist = tmpList;
	playlistCount--;
}

//---------------------------------------
// Initialize the Playlist Table structure
//---------------------------------------

void InitPlaylistTable(TableType *table) {
	UInt32 i;

	for(i = 0; i < playlistVisibleRows; i++) {
		TblSetItemStyle(table, i, 0, checkboxTableItem);
		TblSetItemStyle(table, i, 1, customTableItem);
	}
	TblSetColumnUsable(table, 0, true);
	TblSetColumnUsable(table, 1, true);
	TblSetCustomDrawProcedure(table, 1, DrawPlaylistRow);
}

//-------------------------------------
// Populate the Playlist Table structure
//-------------------------------------

void PopulatePlaylistTable(TableType *table, UInt32 playlistPos, Boolean draw) {
	MemHandle tItem;
	TrackRecordPacked *tRecPacked;
	UInt32 i;

	for(i = 0; i < playlistVisibleRows; i++) {
		if(playlistPos < playlistCount) {
			tItem = DmQueryRecord(palantirDB, playlist[playlistPos]);
			tRecPacked = (TrackRecordPacked *) MemHandleLock(tItem);
			MemHandleUnlock(tItem);
			TblSetItemInt(table, i, 0, 1);
			TblSetItemPtr(table, i, 1, tRecPacked);
			TblSetItemInt(table, i, 1, playlist[playlistPos]);
			playlistPos++;
		} else {
			TblSetRowUsable(table, i, false);
		}
	}

	if(draw) TblDrawTable(table);
}

//-------------------------
// Scroll the Playlist Table
//-------------------------

void ScrollPlaylistTable(Int32 newValue, Boolean setBar) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, PlaylistTable));
	Int32 maxValue = playlistCount - playlistVisibleRows;
	if(maxValue < 0)
		maxValue = 0;
	//make sure we don't scroll above 0 or below max
	if(newValue > maxValue)
		newValue = maxValue;
	if(newValue < 0)
		newValue = 0;

	playlistTablePosition = newValue;
	PopulatePlaylistTable(table, playlistTablePosition, true);
	if(setBar)
		SclSetScrollBar(playlistScrollBar, playlistTablePosition, 0, maxValue, playlistVisibleRows);
}

//----------------------------------------
// Initialize the Playlist Table scrollbars
//----------------------------------------

void InitPlaylistScrollBar() {
	Int16 max;
	if(playlistVisibleRows > playlistCount)
		max = 0;
	else
		max = playlistCount - playlistVisibleRows;
	SclSetScrollBar(playlistScrollBar, 0, 0, max, playlistVisibleRows);
}

//----------------------------
// Initialize the Playlist View
//----------------------------

void PlaylistFormInit(void) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, PlaylistTable));

	//set up some global variables
	playlistTablePosition = 0;				//first _INDEX_ of offset[] array
	playlistVisibleRows = TblGetNumberOfRows(table);	//set the number of rows in the table

	//set the playlist scroll bar object
	playlistScrollBar = (ScrollBarType *) FrmGetObjectPtr(form, FrmGetObjectIndex(form, PlaylistScrollBar));

	if(playlistCount == 0) {

	} else {
		InitPlaylistTable(table);
		InitPlaylistScrollBar();
		PopulatePlaylistTable(table, playlistTablePosition, false);
	}
	FrmDrawForm(form);
}

//---------------------------
// Handle events in BrowseForm
//---------------------------

Boolean PlaylistFormHandleEvent(EventType *event) {
	Int16 row, column;
	TableType *table;
	Boolean handled = false;

	switch(event->eType) {
	case frmOpenEvent:
		//FrmDrawForm(form);
		PlaylistFormInit();
		handled = true;
		break;
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		case PlaylistButtonBack:
			switch(lastScreen) {
				case ARTIST_SCREEN:
					FrmGotoForm(ArtistForm);
					break;
				case SOURCE_SCREEN:
					FrmGotoForm(SourceForm);
					break;
				case TRACK_SCREEN:
					FrmGotoForm(TrackForm);
					break;
				default:
					FrmGotoForm(TrackForm);
					break;
			}
			handled = true;
			break;
		case PlaylistButtonBeam:
			PlaylistBeam();
			break;
		default:
			break;
		}
		break;
	case menuEvent:
		handled = GlobalMenuHandleEvent(event->data.menu.itemID);
		break;
	case sclRepeatEvent:
		ScrollPlaylistTable(event->data.sclExit.newValue, false);
		break;
	case keyDownEvent:
		if(event->data.keyDown.chr == vchrPageDown)
			ScrollPlaylistTable(playlistTablePosition + playlistVisibleRows, true);
		if(event->data.keyDown.chr == vchrPageUp)
			ScrollPlaylistTable(playlistTablePosition - playlistVisibleRows, true);
		handled = true;
		break;
	case tblSelectEvent:
		row = event->data.tblEnter.row;
		column = event->data.tblEnter.column;
		table = event->data.tblEnter.pTable;
		switch(column) {
		case 0:
			//if(TblGetItemInt(table, row, 0) == 1) //checkbox was off, now it's on
			//	PlaylistAdd(TblGetItemInt(table, row, 1));
			//else //checkbox was on, now it's off
			PlaylistRemove(TblGetItemInt(table, row, 1));
			PopulatePlaylistTable(table, playlistTablePosition, true);
			break;
		default:
			break;
		}
		handled = true;
		break;

	default:
		break;
	}
	return handled;
}
