#include <PalmOS.h>
#include <Table.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

//---------------------------------------
// Initialize the Source Table structure
//---------------------------------------

void InitSourceTable(TableType *table) {
	UInt32 i;

	for(i = 0; i < sourceVisibleRows; i++)
		TblSetItemStyle(table, i, 0, customTableItem);
	TblSetColumnUsable(table, 0, true);
	TblSetCustomDrawProcedure(table, 0, DrawSourceRow);
}

//-------------------------------------
// Populate the Source Table structure
//-------------------------------------

void PopulateSourceTable(TableType *table, UInt32 offsetIndex, Boolean draw) {
	MemHandle tItem;
	TrackRecordPacked *tRecPacked;
	UInt32 i, *trackPos;

	trackPos = (UInt32 *)artistSelected->data;

	for(i = 0; i < sourceVisibleRows; i++) {
		if(offsetIndex < artistSelected->sourceCount) {
			tItem = DmQueryRecord(palantirDB, trackPos[offsetIndex] + DBTrackRecordStart);
			tRecPacked = (TrackRecordPacked *) MemHandleLock(tItem);
			MemHandleUnlock(tItem);
			TblSetItemPtr(table, i, 0, tRecPacked);
			TblSetItemInt(table, i, 0, offsetIndex);
			offsetIndex++;
		} else {
			TblSetRowUsable(table, i, false);
		}
	}

	if(draw) TblDrawTable(table);
}

//-------------------------
// Scroll the Source Table
//-------------------------

void ScrollSourceTable(Int32 newValue, Boolean setBar) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, SourceTable));
	Int32 maxValue = artistSelected->sourceCount - sourceVisibleRows;
	if(maxValue < 0)
		maxValue = 0;
	//make sure we don't scroll above 0 or below max
	if(newValue > maxValue)
		newValue = maxValue;
	if(newValue < 0)
		newValue = 0;

	sourceTablePosition = newValue;
	PopulateSourceTable(table, sourceTablePosition, true);
	if(setBar)
		SclSetScrollBar(sourceScrollBar, sourceTablePosition, 0, maxValue, sourceVisibleRows);
}

//----------------------------------------
// Initialize the Source Table scrollbars
//----------------------------------------

void InitSourceScrollBar() {
	Int16 max;
	if(sourceVisibleRows > artistSelected->sourceCount)
		max = 0;
	else
		max = artistSelected->sourceCount - sourceVisibleRows;
	SclSetScrollBar(sourceScrollBar, sourceTablePosition, 0, max, sourceVisibleRows);
}

//----------------------------
// Initialize the Source View
//----------------------------

void SourceFormInit(void) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, SourceTable));

	//set up some global variables
	lastScreen = SOURCE_SCREEN;
	sourceVisibleRows = TblGetNumberOfRows(table);	//set the number of rows in the table
	trackTablePosition = 0;				//set the track number at the top
	//set the source scroll bar object
	sourceScrollBar = (ScrollBarType *) FrmGetObjectPtr(form, FrmGetObjectIndex(form, SourceScrollBar));

	InitSourceTable(table);
	InitSourceScrollBar();
	PopulateSourceTable(table, sourceTablePosition, false);
	FrmDrawForm(form);
}

//---------------------------
// Handle events in BrowseForm
//---------------------------

Boolean SourceFormHandleEvent(EventType *event) {
	Boolean handled = false;
	Int16 row;
	TableType *table;

	switch(event->eType) {
	case frmOpenEvent:
		//FrmDrawForm(form);
		SourceFormInit();
		handled = true;
		break;
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		case SourceButtonBack:
			FrmGotoForm(ArtistForm);
			handled = true;
			break;
		case SourceButtonPlaylist:
			FrmGotoForm(PlaylistForm);
			handled = true;
			break;
		default:
			break;
		}
		break;
	case menuEvent:
		handled = GlobalMenuHandleEvent(event->data.menu.itemID);
		break;
	case sclRepeatEvent:
		ScrollSourceTable(event->data.sclExit.newValue, false);
		break;
	case keyDownEvent:
		if(event->data.keyDown.chr == vchrPageDown)
			ScrollSourceTable(sourceTablePosition + sourceVisibleRows, true);
		if(event->data.keyDown.chr == vchrPageUp)
			ScrollSourceTable(sourceTablePosition - sourceVisibleRows, true);
		handled = true;
		break;
	case tblSelectEvent:
		row = event->data.tblEnter.row;
		table = event->data.tblEnter.pTable;
		sourceSelected = TblGetItemInt(table, row, 0);
		FrmGotoForm(TrackForm);
		handled = true;
		break;
	default:
		break;
	}
	return handled;
}
