#include <PalmOS.h>
#include <Table.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

void DrawArtistRow(void *table, Int16 row, Int16 column, RectangleType *bounds) {
	ArtistRecord *aRec = TblGetItemPtr(table, row, column);
	Char *name = aRec->data + (4 * aRec->sourceCount);
	Char charNumSongs[5];
	FontID oldFont;
	
	StrIToA(charNumSongs, aRec->trackCount);

	oldFont = FntSetFont(stdFont);
	WinDrawTruncChars(name, StrLen(name),
			  bounds->topLeft.x,
			  bounds->topLeft.y,
			  (142 - FntCharsWidth(charNumSongs, StrLen(charNumSongs))));
	FntSetFont(boldFont);
	WinDrawTruncChars(charNumSongs, StrLen(charNumSongs),
			  bounds->topLeft.x + (150 - FntCharsWidth(charNumSongs, StrLen(charNumSongs))),
			  bounds->topLeft.y,
			  FntCharsWidth(charNumSongs, StrLen(charNumSongs)));
	FntSetFont(oldFont);
}

void DrawSourceRow(void *table, Int16 row, Int16 column, RectangleType *bounds) {
	TrackRecordPacked *tRecPacked = TblGetItemPtr(table, row, column);
	TrackRecord tRec;
	FontID oldFont;
	Char charNumSongs[5];	
	UInt32 *posArray = (UInt32 *)artistSelected->data;
	UInt32 offsetIndex = TblGetItemInt(table, row, column);
	UInt32 nextPos, thisPos = posArray[offsetIndex];
	
	nextPos = (offsetIndex + 1) == artistSelected->sourceCount ?
				    posArray[0] + artistSelected->trackCount :
				    posArray[offsetIndex + 1];
	
	StrIToA(charNumSongs, nextPos - thisPos);
	UnpackTrackRecord(&tRec, tRecPacked);
	
	oldFont = FntSetFont(stdFont);
	WinSetUnderlineMode(noUnderline);
	WinDrawTruncChars(tRec.source, StrLen(tRec.source),
			  bounds->topLeft.x,
			  bounds->topLeft.y, 
			  (142 - FntCharsWidth(charNumSongs, StrLen(charNumSongs))));
	FntSetFont(boldFont);
	WinDrawTruncChars(charNumSongs, StrLen(charNumSongs),
			  bounds->topLeft.x + (150 - FntCharsWidth(charNumSongs, StrLen(charNumSongs))),
			  bounds->topLeft.y,
			  FntCharsWidth(charNumSongs, StrLen(charNumSongs)));
	FntSetFont(oldFont);
}

void DrawTrackRow(void *table, Int16 row, Int16 column, RectangleType *bounds) {
	TrackRecordPacked *tRecPacked = TblGetItemPtr(table, row, column);
	TrackRecord tRec;
	FontID oldFont;
	
	UnpackTrackRecord(&tRec, tRecPacked);
	
	oldFont = FntSetFont(stdFont);
	WinSetUnderlineMode(noUnderline);
	WinDrawTruncChars(tRec.title, StrLen(tRec.title),
			  bounds->topLeft.x,
			  bounds->topLeft.y,
			  (128 - FntCharsWidth(tRec.duration, StrLen(tRec.duration))));
	FntSetFont(boldFont);
	WinDrawTruncChars(tRec.duration, StrLen(tRec.duration),
			  bounds->topLeft.x + (136 - FntCharsWidth(tRec.duration, StrLen(tRec.duration))),
			  bounds->topLeft.y,
			  FntCharsWidth(tRec.duration, StrLen(tRec.duration)));
	FntSetFont(oldFont);
}

void DrawPlaylistRow(void *table, Int16 row, Int16 column, RectangleType *bounds) {
	TrackRecordPacked *tRecPacked = TblGetItemPtr(table, row, column);
	TrackRecord tRec;
	FontID oldFont;
	
	UnpackTrackRecord(&tRec, tRecPacked);
	
	oldFont = FntSetFont(stdFont);
	WinSetUnderlineMode(noUnderline);
	WinDrawTruncChars(tRec.title, StrLen(tRec.title),
			  bounds->topLeft.x,
			  bounds->topLeft.y,
			  (128 - FntCharsWidth(tRec.duration, StrLen(tRec.duration))));
	FntSetFont(boldFont);
	WinDrawTruncChars(tRec.duration, StrLen(tRec.duration),
			  bounds->topLeft.x + (136 - FntCharsWidth(tRec.duration, StrLen(tRec.duration))),
			  bounds->topLeft.y,
			  FntCharsWidth(tRec.duration, StrLen(tRec.duration)));
	FntSetFont(oldFont);
}
