#include <PalmOS.h>
#include <Table.h>
#include "palantirResource.h"
#include "palantirGlobals.h"

//---------------------------------------
// Initialize the Track Table structure
//---------------------------------------

void InitTrackTable(TableType *table) {
	UInt32 i;

	for(i = 0; i < trackVisibleRows; i++) {
		TblSetItemStyle(table, i, 0, checkboxTableItem);
		//TblSetItemInt(table, i, 0, 0);
		TblSetItemStyle(table, i, 1, customTableItem);
	}
	TblSetColumnUsable(table, 0, true);
	TblSetColumnUsable(table, 1, true);
	TblSetCustomDrawProcedure(table, 1, DrawTrackRow);
}

//-------------------------------------
// Populate the Track Table structure
//-------------------------------------

void PopulateTrackTable(TableType *table, UInt32 trackPos, Boolean draw) {
	MemHandle tItem;
	TrackRecordPacked *tRecPacked;
	UInt32 i, index, *posArray = (UInt32 *)artistSelected->data;

	for(i = 0; i < trackVisibleRows; i++) {
		if(trackPos < trackTotalCount) {
			index = trackPos + posArray[sourceSelected] + DBTrackRecordStart;
			tItem = DmQueryRecord(palantirDB, index);
			tRecPacked = (TrackRecordPacked *) MemHandleLock(tItem);
			MemHandleUnlock(tItem);
			TblSetItemInt(table, i, 0, ((PlaylistContains(index) == -1) ? 0 : 1));
			TblSetItemPtr(table, i, 1, tRecPacked);
			TblSetItemInt(table, i, 1, index);
			trackPos++;
		} else {
			TblSetRowUsable(table, i, false);
		}
	}
	
	if(draw) TblDrawTable(table);
}

//-------------------------
// Scroll the Track Table
//-------------------------

void ScrollTrackTable(Int32 newValue, Boolean setBar) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, TrackTable));
	Int32 maxValue = trackTotalCount - trackVisibleRows;
	if(maxValue < 0)
		maxValue = 0;
	//make sure we don't scroll above 0 or below max
	if(newValue > maxValue)
		newValue = maxValue;
	if(newValue < 0)
		newValue = 0;
		
	trackTablePosition = newValue;
	PopulateTrackTable(table, trackTablePosition, true);
	if(setBar)
		SclSetScrollBar(trackScrollBar, trackTablePosition, 0, maxValue, trackVisibleRows);
}

//----------------------------------------
// Initialize the Track Table scrollbars
//----------------------------------------

void InitTrackScrollBar() {
	Int16 max;
	if(trackVisibleRows > trackTotalCount)
		max = 0;
	else
		max = trackTotalCount - trackVisibleRows;
	SclSetScrollBar(trackScrollBar, trackTablePosition, 0, max, trackVisibleRows);
}

//----------------------------
// Initialize the Track View
//----------------------------

void TrackFormInit(void) {
	FormType *form = FrmGetActiveForm();
	TableType *table = FrmGetObjectPtr(form, FrmGetObjectIndex(form, TrackTable));
	UInt32 firstTrack, lastTrack, *posArray;

	//set up some global variables
	lastScreen = TRACK_SCREEN;
	trackVisibleRows = TblGetNumberOfRows(table);	//set the number of rows in the table
	
	posArray = (UInt32 *)artistSelected->data;
	firstTrack = posArray[sourceSelected];
	lastTrack = (sourceSelected + 1) == artistSelected->sourceCount ?
				         posArray[0] + artistSelected->trackCount :
				         posArray[sourceSelected + 1];
	trackTotalCount = lastTrack - firstTrack;
	
	//set the track scroll bar object
	trackScrollBar = (ScrollBarType *) FrmGetObjectPtr(form, FrmGetObjectIndex(form, TrackScrollBar));

	InitTrackTable(table);
	InitTrackScrollBar();
	PopulateTrackTable(table, trackTablePosition, false);
	FrmDrawForm(form);
}

//---------------------------
// Handle events in BrowseForm
//---------------------------

Boolean TrackFormHandleEvent(EventType *event) {
	Int16 row, column;
	TableType *table;
	Boolean handled = false;

	switch(event->eType) {
	case frmOpenEvent:
		//FrmDrawForm(form);
		TrackFormInit();
		handled = true;
		break;
	case ctlSelectEvent:
		switch(event->data.ctlSelect.controlID) {
		case TrackButtonBack:
			FrmGotoForm(SourceForm);
			handled = true;
			break;
		case TrackButtonPlaylist:
			FrmGotoForm(PlaylistForm);
			handled = true;
			break;
		default:
			break;
		}
		break;
	case menuEvent:
		handled = GlobalMenuHandleEvent(event->data.menu.itemID);
		break;
		case sclRepeatEvent:
			ScrollTrackTable(event->data.sclExit.newValue, false);
			break;
		case keyDownEvent:
			if(event->data.keyDown.chr == vchrPageDown)
				ScrollTrackTable(trackTablePosition + trackVisibleRows, true);
			if(event->data.keyDown.chr == vchrPageUp)
				ScrollTrackTable(trackTablePosition - trackVisibleRows, true);
			handled = true;
			break;
	case tblSelectEvent:
		row = event->data.tblEnter.row;
		column = event->data.tblEnter.column;
		table = event->data.tblEnter.pTable;
		switch(column) {
		case 0:
			if(TblGetItemInt(table, row, 0) == 1) //checkbox was off, now it's on
				PlaylistAdd(TblGetItemInt(table, row, 1));
			else //checkbox was on, now it's off
				PlaylistRemove(TblGetItemInt(table, row, 1));
			break;
		default:
			break;
		}
		handled = true;
		break;
	default:
		break;
	}
	return handled;
}
